'use strict';

let makeGithubRequest = (() => {
  var _ref = _asyncToGenerator(function* (uri, body, token, method = 'POST') {
    return (0, _requestPromiseNative2.default)({
      method,
      uri,
      body,
      json: true,
      headers: {
        'User-Agent': 'cloudbuild-github'
      },
      auth: {
        user: 'token',
        pass: token
      }
    });
  });

  return function makeGithubRequest(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
})();

let deploy = (() => {
  var _ref2 = _asyncToGenerator(function* () {
    const ui = new _inquirer2.default.ui.BottomBar();

    const projectId = yield _gcloud.googleApis.getProjectId();

    const webhookSecret = yield _keymanager.keyManager.getWebhookSecret();

    ui.log.write('Deploying cloud functions.');
    if (_commander2.default.delete) {
      try {
        yield (0, _childProcessPromise.spawn)('gcloud', ['--quiet', 'beta', 'functions', 'delete', 'cloudbuildGithubWebhook'], { stdio: 'inherit' });
      } catch (err) {
        ui.log.write('Could not delete cloudbuildGithubWebhook, skipping.');
      }
      try {
        yield (0, _childProcessPromise.spawn)('gcloud', ['--quiet', 'beta', 'functions', 'delete', 'cloudbuildGithubNotifier'], { stdio: 'inherit' });
      } catch (err) {
        ui.log.write('Could not delete cloudbuildGithubNotifier, skipping.');
      }
    }
    yield (0, _childProcessPromise.spawn)('gcloud', ['beta', 'functions', 'deploy', 'cloudbuildGithubWebhook', '--trigger-http'], { stdio: 'inherit' });
    yield (0, _childProcessPromise.spawn)('gcloud', ['beta', 'functions', 'deploy', 'cloudbuildGithubNotifier', '--trigger-topic', 'cloud-builds'], { stdio: 'inherit' });

    const webhookUrl = `https://us-central1-${projectId}.cloudfunctions.net/cloudbuildGithubWebhook`;

    yield Promise.all(Object.keys(_config2.default.repos).map((() => {
      var _ref3 = _asyncToGenerator(function* (repoName) {
        const repoToken = yield _keymanager.keyManager.getGithubToken(repoName);
        ui.log.write(`Setting up repository webhook for ${repoName}`);
        const hooksUri = `${GITHUB_API_BASE}/repos/${repoName}/hooks`;
        const existingHooks = yield makeGithubRequest(hooksUri, null, repoToken, 'GET');
        yield Promise.all(existingHooks.filter(function (hook) {
          return hook.config.url === webhookUrl;
        }).map(function (hook) {
          return makeGithubRequest(`${hooksUri}/${hook.id}`, null, repoToken, 'DELETE');
        }));

        const hook = {
          name: 'web',
          config: {
            url: webhookUrl,
            content_type: 'json',
            secret: webhookSecret
          },
          events: ['pull_request']
        };
        yield makeGithubRequest(hooksUri, hook, repoToken);
      });

      return function (_x4) {
        return _ref3.apply(this, arguments);
      };
    })()));

    ui.log.write('Done!');
  });

  return function deploy() {
    return _ref2.apply(this, arguments);
  };
})();

var _process = require('process');

var _process2 = _interopRequireDefault(_process);

var _childProcessPromise = require('child-process-promise');

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _inquirer = require('inquirer');

var _inquirer2 = _interopRequireDefault(_inquirer);

var _requestPromiseNative = require('request-promise-native');

var _requestPromiseNative2 = _interopRequireDefault(_requestPromiseNative);

var _package = require('../../package.json');

var _package2 = _interopRequireDefault(_package);

var _config = require('../config');

var _config2 = _interopRequireDefault(_config);

var _gcloud = require('../gcloud');

var _keymanager = require('../keymanager');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * MIT License
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Copyright (c) 2018 Choko (choko@curioswitch.org)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Permission is hereby granted, free of charge, to any person obtaining a copy
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * of this software and associated documentation files (the "Software"), to deal
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * in the Software without restriction, including without limitation the rights
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * copies of the Software, and to permit persons to whom the Software is
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * furnished to do so, subject to the following conditions:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * The above copyright notice and this permission notice shall be included in all
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * copies or substantial portions of the Software.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * SOFTWARE.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            */

const GITHUB_API_BASE = 'https://api.github.com';

_commander2.default.version(_package2.default.version).option('--delete', 'Delete existing functions first').parse(_process2.default.argv);

deploy().then(() => _process2.default.exit(), err => {
  console.error('Unexpected error.', err);
  _process2.default.exit(1);
});