'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.handleBuildEvent = undefined;

let makeRequest = (() => {
  var _ref = _asyncToGenerator(function* (uri, body) {
    const repo = GITHUB_URL_REPO_REGEX.exec(uri)[1];
    const githubToken = yield _keymanager.keyManager.getGithubToken(repo);
    return (0, _requestPromiseNative2.default)({
      method: 'POST',
      uri,
      body,
      json: true,
      headers: {
        'User-Agent': 'cloudbuild-github'
      },
      auth: {
        user: 'token',
        pass: githubToken
      }
    });
  });

  return function makeRequest(_x, _x2) {
    return _ref.apply(this, arguments);
  };
})();

let handleBuildEvent = exports.handleBuildEvent = (() => {
  var _ref2 = _asyncToGenerator(function* (event) {
    const build = JSON.parse(Buffer.from(event.data.data, 'base64').toString('utf8'));

    let repoName = null;
    let revisionId = null;
    if (!build.substitutions) {
      // No substitutions for triggered builds, we'll post to the commit instead.
      const repoSource = build.sourceProvenance.resolvedRepoSource;
      // eslint-disable-next-line prefer-destructuring
      const cloudbuildRepoName = repoSource.repoName;
      if (!cloudbuildRepoName.startsWith('github-')) {
        // Not a github repo.
        return;
      }
      repoName = cloudbuildRepoName.substring('github-'.length).replace('-', '/');
      revisionId = repoSource.commitSha;
    }

    const statusesUrl = repoName && revisionId ? `${GITHUB_API_BASE}/repos/${repoName}/statuses/${revisionId}` : build.substitutions[_constants.STATUSES_URL_KEY];
    if (!statusesUrl) {
      // A non-triggered build not from the webhook, nothing to do.
      return;
    }

    const status = {
      state: statusToState(build.status),
      target_url: build.logUrl,
      description: statusToDescription(build.status),
      context: 'ci/cloudbuild'
    };

    const statusResponse = yield makeRequest(statusesUrl, status);
    if (!statusResponse.state) {
      throw new Error(`Failed to set status: ${JSON.stringify(statusResponse)}`);
    }

    const commentsUrl = repoName && revisionId ? `${GITHUB_API_BASE}/repos/${repoName}/commits/${revisionId}/comments` : build.substitutions[_constants.COMMENTS_URL_KEY];
    if (!commentsUrl) {
      // A non-triggered build not from the webhook, nothing to do (we shouldn't
      // actually get here since we have a similar check for statuses).
      return;
    }

    let comment;
    switch (build.status) {
      case 'QUEUED':
      case 'WORKING':
      case 'CANCELLED':
      case 'STATUS_UNKNOWN':
        return;
      case 'SUCCESS':
        {
          if (repoName) {
            // Don't comment on success for triggered builds.
            return;
          }
          comment = `Build succeded. If you have approval, you're ready to merge!\n\nLogs:\n${build.logUrl}`;
          break;
        }
      case 'FAILURE':
        comment = `Build failed. Check the logs and try again.\n\nLogs:\n${build.logUrl}`;
        break;
      default:
        comment = `Build terminated with unknown error. You may want to retry.\n\nLogs:\n${build.logUrl}`;
        break;
    }
    const commentResponse = yield makeRequest(commentsUrl, { body: comment });
    if (!commentResponse.id) {
      throw new Error(`Failed to set comment: ${JSON.stringify(commentResponse)}`);
    }
  });

  return function handleBuildEvent(_x3) {
    return _ref2.apply(this, arguments);
  };
})();

var _requestPromiseNative = require('request-promise-native');

var _requestPromiseNative2 = _interopRequireDefault(_requestPromiseNative);

var _keymanager = require('./keymanager');

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; } /*
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * MIT License
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Copyright (c) 2017 Choko (choko@curioswitch.org)
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * Permission is hereby granted, free of charge, to any person obtaining a copy
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * of this software and associated documentation files (the "Software"), to deal
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * in the Software without restriction, including without limitation the rights
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * copies of the Software, and to permit persons to whom the Software is
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * furnished to do so, subject to the following conditions:
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * The above copyright notice and this permission notice shall be included in all
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * copies or substantial portions of the Software.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            *
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            * SOFTWARE.
                                                                                                                                                                                                                                                                                                                                                                                                                                                                            */

const GITHUB_API_BASE = 'https://api.github.com';

const GITHUB_URL_REPO_REGEX = /repos\/([^/]+\/[^/]+)\//;

function statusToState(status) {
  switch (status) {
    case 'QUEUED':
    case 'WORKING':
      return 'pending';
    case 'SUCCESS':
      return 'success';
    case 'FAILURE':
      return 'failure';
    default:
      return 'error';
  }
}

function statusToDescription(status) {
  switch (status) {
    case 'QUEUED':
      return 'Build queued.';
    case 'WORKING':
      return 'Build started.';
    case 'SUCCESS':
      return 'Build succeeded.';
    case 'FAILURE':
      return 'Build failed.';
    case 'INTERNAL_ERROR':
      return 'Build internal error.';
    case 'TIMEOUT':
      return 'Build timed out.';
    case 'CANCELLED':
      return 'Build cancelled.';
    default:
      return 'Build status unknown.';
  }
}